/* Current database is 'testdb' */
\c testdb postgres
CREATE OR REPLACE FUNCTION chkrolattr()
 RETURNS TABLE (name name, label text, canlogin bool, replication bool)
 AS $$
SELECT r.rolname, v.label, r.rolcanlogin, r.rolreplication
 FROM pg_roles r
 JOIN (VALUES(CURRENT_USER, 'current_user'),
             (SESSION_USER, 'session_user'))
      AS v(uname, label)
      ON (r.rolname = v.uname)
 ORDER BY 1;
$$ LANGUAGE SQL;

CREATE OR REPLACE FUNCTION chksetconfig()
 RETURNS TABLE (name name, label text, setconfig text[])
 AS $$
SELECT r.rolname, v.label, s.setconfig
 FROM pg_roles r
 JOIN pg_db_role_setting s ON (r.oid = s.setrole)
 JOIN (VALUES(CURRENT_USER, 'current_user'),
             (SESSION_USER, 'session_user'))
      AS v(uname, label)
      ON (r.rolname = v.uname)
 WHERE s.setdatabase = 0
UNION
SELECT 'ALL', 'ALL', s.setconfig
 FROM pg_db_role_setting s
 WHERE s.setdatabase = 0 AND s.setrole = 0
ORDER BY 1;
$$ LANGUAGE SQL;

CREATE OR REPLACE FUNCTION chkumapping()
 RETURNS TABLE (umname name, umserver name, umoptions text[])
 AS $$
SELECT r.rolname, s.srvname, m.umoptions
 FROM pg_user_mapping m
 LEFT JOIN pg_roles r ON (r.oid = m.umuser)
 JOIN pg_foreign_server s ON (s.oid = m.umserver)
 ORDER BY 2;
$$ LANGUAGE SQL;

CREATE ROLE testrol0 WITH SUPERUSER LOGIN;
CREATE ROLE testrolx WITH SUPERUSER LOGIN;
CREATE ROLE testrol2 WITH SUPERUSER;
CREATE ROLE testrol1 WITH SUPERUSER LOGIN IN ROLE testrol2;
\c testdb testrol1
SET ROLE testrol2;


--  ALTER ROLE
SELECT * FROM chkrolattr();
ALTER ROLE CURRENT_ROLE WITH REPLICATION;
SELECT * FROM chkrolattr();
ALTER ROLE CURRENT_USER WITH NOREPLICATION;
SELECT * FROM chkrolattr();
ALTER ROLE USER WITH REPLICATION;
SELECT * FROM chkrolattr();
ALTER ROLE SESSION_USER WITH REPLICATION;
SELECT * FROM chkrolattr();
ALTER ROLE testrol1 WITH NOREPLICATION;
ALTER ROLE testrol2 WITH NOREPLICATION;
SELECT * FROM chkrolattr();

ALTER ROLE ALL WITH REPLICATION; -- error
ALTER ROLE SESSION_ROLE WITH NOREPLICATION; -- error
ALTER ROLE PUBLIC WITH NOREPLICATION; -- error
ALTER ROLE nonexistent WITH NOREPLICATION; -- error

--  ALTER USER
SELECT * FROM chkrolattr();
ALTER USER CURRENT_ROLE WITH REPLICATION;
SELECT * FROM chkrolattr();
ALTER USER CURRENT_USER WITH NOREPLICATION;
SELECT * FROM chkrolattr();
ALTER USER USER WITH REPLICATION;
SELECT * FROM chkrolattr();
ALTER USER SESSION_USER WITH REPLICATION;
SELECT * FROM chkrolattr();
ALTER USER testrol1 WITH NOREPLICATION;
ALTER USER testrol2 WITH NOREPLICATION;
SELECT * FROM chkrolattr();

ALTER USER ALL WITH REPLICATION; -- error
ALTER USER SESSION_ROLE WITH NOREPLICATION; -- error
ALTER USER PUBLIC WITH NOREPLICATION; -- error
ALTER USER nonexistent WITH NOREPLICATION; -- error

--  ALTER ROLE SET/RESET
SELECT * FROM chksetconfig();
ALTER ROLE CURRENT_USER SET application_name to 'FOO';
ALTER ROLE SESSION_USER SET application_name to 'BAR';
SELECT * FROM chksetconfig();
ALTER ROLE CURRENT_USER RESET application_name;
ALTER ROLE SESSION_USER RESET application_name;
SELECT * FROM chksetconfig();

ALTER ROLE CURRENT_ROLE SET application_name to 'BAZ';
SELECT * FROM chksetconfig();
ALTER ROLE CURRENT_ROLE RESET application_name;
SELECT * FROM chksetconfig();

ALTER ROLE USER SET application_name to 'BOOM';
SELECT * FROM chksetconfig();
ALTER ROLE USER RESET application_name;
SELECT * FROM chksetconfig();

ALTER ROLE testrol1 SET application_name to 'SLAM';
SELECT * FROM chksetconfig();
ALTER ROLE testrol1 RESET application_name;
SELECT * FROM chksetconfig();

ALTER ROLE ALL SET application_name to 'SLAP';
SELECT * FROM chksetconfig();
ALTER ROLE ALL RESET application_name;
SELECT * FROM chksetconfig();

ALTER ROLE PUBLIC SET application_name to 'BOMB'; -- error
ALTER ROLE nonexistent SET application_name to 'BOMB'; -- error
ALTER ROLE PUBLIC RESET application_name; -- error
ALTER ROLE nonexistent RESET application_name; -- error

--  ALTER USER SET/RESET
SELECT * FROM chksetconfig();
ALTER USER CURRENT_USER SET application_name to 'FOO';
ALTER USER SESSION_USER SET application_name to 'BAR';
SELECT * FROM chksetconfig();
ALTER USER CURRENT_USER RESET application_name;
ALTER USER SESSION_USER RESET application_name;
SELECT * FROM chksetconfig();

ALTER USER CURRENT_ROLE SET application_name to 'BAZ';
SELECT * FROM chksetconfig();
ALTER USER CURRENT_ROLE RESET application_name;
SELECT * FROM chksetconfig();

ALTER USER USER SET application_name to 'BOOM';
SELECT * FROM chksetconfig();
ALTER USER USER RESET application_name;
SELECT * FROM chksetconfig();

ALTER USER testrol1 SET application_name to 'SLAM';
SELECT * FROM chksetconfig();
ALTER USER testrol1 RESET application_name;
SELECT * FROM chksetconfig();

ALTER USER ALL SET application_name to 'SLAP';
SELECT * FROM chksetconfig();
ALTER USER ALL RESET application_name;
SELECT * FROM chksetconfig();

ALTER USER PUBLIC SET application_name to 'BOMB'; -- error
ALTER USER nonexistent SET application_name to 'BOMB'; -- error
ALTER USER PUBLIC RESET application_name; -- error
ALTER USER nonexistent RESET application_name; -- error

-- CREAETE SCHEMA
set client_min_messages to error;
CREATE SCHEMA newschema1 AUTHORIZATION CURRENT_USER;
CREATE SCHEMA newschema2 AUTHORIZATION CURRENT_ROLE;
CREATE SCHEMA newschema3 AUTHORIZATION USER;
CREATE SCHEMA newschema4 AUTHORIZATION SESSION_USER;
CREATE SCHEMA newschema5 AUTHORIZATION testrolx;

CREATE SCHEMA newschema6 AUTHORIZATION PUBLIC; -- error
CREATE SCHEMA newschema6 AUTHORIZATION nonexistent; -- error

SELECT n.nspname, r.rolname FROM pg_namespace n
 JOIN pg_roles r ON (r.oid = n.nspowner)
 WHERE n.nspname LIKE 'newschema_' ORDER BY 1;

DROP SCHEMA IF EXISTS newschema1;
DROP SCHEMA IF EXISTS newschema2;
DROP SCHEMA IF EXISTS newschema3;
DROP SCHEMA IF EXISTS newschema4;
DROP SCHEMA IF EXISTS newschema5;
DROP SCHEMA IF EXISTS newschema6;

CREATE SCHEMA IF NOT EXISTS newschema1 AUTHORIZATION CURRENT_USER;
CREATE SCHEMA IF NOT EXISTS newschema2 AUTHORIZATION CURRENT_ROLE;
CREATE SCHEMA IF NOT EXISTS newschema3 AUTHORIZATION USER;
CREATE SCHEMA IF NOT EXISTS newschema4 AUTHORIZATION SESSION_USER;
CREATE SCHEMA IF NOT EXISTS newschema5 AUTHORIZATION testrolx;

CREATE SCHEMA IF NOT EXISTS newschema6 AUTHORIZATION PUBLIC; -- error
CREATE SCHEMA IF NOT EXISTS newschema6 AUTHORIZATION nonexistent; -- error

SELECT n.nspname, r.rolname FROM pg_namespace n
 JOIN pg_roles r ON (r.oid = n.nspowner)
 WHERE n.nspname LIKE 'newschema_' ORDER BY 1;

-- ALTER TABLE OWNER TO
\c testdb testrol0
set client_min_messages to error;
CREATE TABLE testtab1 (a int);
CREATE TABLE testtab2 (a int);
CREATE TABLE testtab3 (a int);
CREATE TABLE testtab4 (a int);
CREATE TABLE testtab5 (a int);
CREATE TABLE testtab6 (a int);
\c testdb testrol1
SET ROLE testrol2;

ALTER TABLE testtab2 OWNER TO CURRENT_USER;
ALTER TABLE testtab3 OWNER TO CURRENT_ROLE;
ALTER TABLE testtab4 OWNER TO USER;
ALTER TABLE testtab5 OWNER TO SESSION_USER;
ALTER TABLE testtab6 OWNER TO testrolx;
ALTER TABLE testtab6 OWNER TO PUBLIC; -- error
ALTER TABLE testtab6 OWNER TO nonexistent; -- error

SELECT c.relname, r.rolname
 FROM pg_class c JOIN pg_roles r ON (r.oid = c.relowner)
 WHERE relname LIKE 'testtab_'
 ORDER BY 1;

-- ALTER AGGREGATE
\c testdb testrol0
CREATE AGGREGATE testagg1(int2) (SFUNC = int2_sum, STYPE = int8);
CREATE AGGREGATE testagg2(int2) (SFUNC = int2_sum, STYPE = int8);
CREATE AGGREGATE testagg3(int2) (SFUNC = int2_sum, STYPE = int8);
CREATE AGGREGATE testagg4(int2) (SFUNC = int2_sum, STYPE = int8);
CREATE AGGREGATE testagg5(int2) (SFUNC = int2_sum, STYPE = int8);
CREATE AGGREGATE testagg6(int2) (SFUNC = int2_sum, STYPE = int8);

\c testdb testrol1
SET ROLE testrol2;

ALTER AGGREGATE testagg1(int2) OWNER TO CURRENT_USER;
ALTER AGGREGATE testagg2(int2) OWNER TO CURRENT_ROLE;
ALTER AGGREGATE testagg3(int2) OWNER TO USER;
ALTER AGGREGATE testagg4(int2) OWNER TO SESSION_USER;
ALTER AGGREGATE testagg5(int2) OWNER TO testrolx;
ALTER AGGREGATE testagg6(int2) OWNER TO PUBLIC; -- error
ALTER AGGREGATE testagg6(int2) OWNER TO nonexistent; -- error

SELECT p.proname, r.rolname
 FROM pg_proc p JOIN pg_roles r ON (r.oid = p.proowner)
 WHERE proname LIKE 'testagg_'
 ORDER BY 1;

-- ALTER COLLATION, CONVERSION, DATABASE, DOMAIN, FUNCTION, LANGUAGE,
--       LARGEOBJECT, OPERATOR, OPCLASS, OPFAMILY, SCHEMA, TYPE,
--       TABLESPACE, TSDICTIONARY, TSCONFIGURATION, FDW, FOREIGN_SERVER,
--       EVENT_TRIGGER are processed in the same way.
-- See ExecAlterOwner Stmt for details.


-- CREATE USER MAPPING
-- needs postgres_fdw
CREATE EXTENSION postgres_fdw;
CREATE SERVER sv1 FOREIGN DATA WRAPPER postgres_fdw;
CREATE SERVER sv2 FOREIGN DATA WRAPPER postgres_fdw;
CREATE SERVER sv3 FOREIGN DATA WRAPPER postgres_fdw;
CREATE SERVER sv4 FOREIGN DATA WRAPPER postgres_fdw;
CREATE SERVER sv5 FOREIGN DATA WRAPPER postgres_fdw;
CREATE SERVER sv6 FOREIGN DATA WRAPPER postgres_fdw;
CREATE SERVER sv7 FOREIGN DATA WRAPPER postgres_fdw;
CREATE USER MAPPING FOR PUBLIC SERVER sv1 OPTIONS (user 'public');
CREATE USER MAPPING FOR CURRENT_USER SERVER sv2 OPTIONS (user 'CURRENT_USER');
CREATE USER MAPPING FOR CURRENT_ROLE SERVER sv3 OPTIONS (user 'CURRENT_ROLE');
CREATE USER MAPPING FOR USER SERVER sv4 OPTIONS (user 'USER');
CREATE USER MAPPING FOR SESSION_USER SERVER sv5 OPTIONS (user 'SESSION_USER');
CREATE USER MAPPING FOR testrolx SERVER sv6 OPTIONS (user 'testrolx');
CREATE USER MAPPING FOR nonexistent SERVER sv7
	    OPTIONS (user 'nonexistent'); -- error;

SELECT * FROM chkumapping();

-- ALTER USER MAPPING
ALTER USER MAPPING FOR PUBLIC SERVER sv1
 OPTIONS (SET user 'public_alt');
ALTER USER MAPPING FOR CURRENT_USER SERVER sv2
 OPTIONS (SET user 'CURRENT_USER_alt');
ALTER USER MAPPING FOR CURRENT_ROLE SERVER sv3
 OPTIONS (SET user 'CURRENT_ROLE_alt');
ALTER USER MAPPING FOR USER SERVER sv4
 OPTIONS (SET user 'USER_alt');
ALTER USER MAPPING FOR SESSION_USER SERVER sv5
 OPTIONS (SET user 'SESSION_USER_alt');
ALTER USER MAPPING FOR testrolx SERVER sv6
 OPTIONS (SET user 'testrolx_alt');
ALTER USER MAPPING FOR nonexistent SERVER sv7
 OPTIONS (SET user 'nonexistent_alt'); -- error

SELECT * FROM chkumapping();

-- DROP USER MAPPING
DROP USER MAPPING FOR PUBLIC SERVER sv1;
SELECT * FROM chkumapping();
DROP USER MAPPING FOR CURRENT_USER SERVER sv2;
SELECT * FROM chkumapping();
DROP USER MAPPING FOR CURRENT_ROLE SERVER sv3;
SELECT * FROM chkumapping();
DROP USER MAPPING FOR USER SERVER sv4;
SELECT * FROM chkumapping();
DROP USER MAPPING FOR SESSION_USER SERVER sv5;
SELECT * FROM chkumapping();
DROP USER MAPPING FOR testrolx SERVER sv6;
SELECT * FROM chkumapping();
DROP USER MAPPING FOR nonexistent SERVER sv7;
SELECT * FROM chkumapping();

CREATE USER MAPPING FOR PUBLIC SERVER sv1 OPTIONS (user 'public');
CREATE USER MAPPING FOR CURRENT_USER SERVER sv2 OPTIONS (user 'CURRENT_USER');
CREATE USER MAPPING FOR CURRENT_ROLE SERVER sv3 OPTIONS (user 'CURRENT_ROLE');
CREATE USER MAPPING FOR USER SERVER sv4 OPTIONS (user 'USER');
CREATE USER MAPPING FOR SESSION_USER SERVER sv5 OPTIONS (user 'SESSION_USER');
CREATE USER MAPPING FOR testrolx SERVER sv6 OPTIONS (user 'testrolx');
SELECT * FROM chkumapping();

-- DROP USER MAPPING IF EXISTS
DROP USER MAPPING IF EXISTS FOR PUBLIC SERVER sv1;
SELECT * FROM chkumapping();
DROP USER MAPPING IF EXISTS FOR CURRENT_USER SERVER sv2;
SELECT * FROM chkumapping();
DROP USER MAPPING IF EXISTS FOR CURRENT_ROLE SERVER sv3;
SELECT * FROM chkumapping();
DROP USER MAPPING IF EXISTS FOR USER SERVER sv4;
SELECT * FROM chkumapping();
DROP USER MAPPING IF EXISTS FOR SESSION_USER SERVER sv5;
SELECT * FROM chkumapping();
DROP USER MAPPING IF EXISTS FOR testrolx SERVER sv6;
SELECT * FROM chkumapping();
DROP USER MAPPING IF EXISTS FOR nonexistent SERVER sv7;
SELECT * FROM chkumapping();

-- GRANT/REVOKE : These are not modified by this patch.
GRANT ALL PRIVILEGES ON FUNCTION testagg1(int2)
 TO PUBLIC; -- no error
GRANT ALL PRIVILEGES ON FUNCTION testagg2(int2)
 TO testrolx; --no error
GRANT ALL PRIVILEGES ON FUNCTION testagg3(int2)
 TO CURRENT_USER; --error
GRANT ALL PRIVILEGES ON FUNCTION testagg4(int2)
 TO CURRENT_ROLE; --error
GRANT ALL PRIVILEGES ON FUNCTION testagg5(int2)
 TO USER; --error
GRANT ALL PRIVILEGES ON FUNCTION testagg6(int2)
 TO SESSION_USER; --error

SELECT proname, proacl FROM pg_proc WHERE proname LIKE 'testagg_';

REVOKE ALL PRIVILEGES ON FUNCTION testagg1(int2)
 FROM PUBLIC; -- no error
REVOKE ALL PRIVILEGES ON FUNCTION testagg2(int2)
 FROM testrolx; --no error
REVOKE ALL PRIVILEGES ON FUNCTION testagg3(int2)
 FROM CURRENT_USER; --error
REVOKE ALL PRIVILEGES ON FUNCTION testagg4(int2)
 FROM CURRENT_ROLE; --error
REVOKE ALL PRIVILEGES ON FUNCTION testagg5(int2)
 FROM USER; --error
REVOKE ALL PRIVILEGES ON FUNCTION testagg6(int2)
 FROM SESSION_USER; --error

SELECT proname, proacl FROM pg_proc WHERE proname LIKE 'testagg_';

